'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _fp = require('lodash/fp');

var _baconjs = require('baconjs');

var _baconjs2 = _interopRequireDefault(_baconjs);

var _shortid = require('shortid');

var _plugins = require('./plugins');

var _envelope = require('./data/envelope');

var _data = require('./data/data');

var _data2 = _interopRequireDefault(_data);

var _utils = require('./utils');

var _combinators = require('./utils/combinators');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// The following functions provide funtionalities that should be run every
// time a plugin is run. The plugin runner composes them with the plugin.
var stats = (0, _fp.curry)(function (stream, name, envelope) {
  stream.push({ type: 'plugin_stats', plugin: name, size: (0, _fp.size)(envelope.data) });
  return envelope;
});

var start = (0, _fp.curry)(function (stream, name, envelope) {
  stream.push({ type: 'plugin_start', ts: (0, _utils.now)(), plugin: name });
  return envelope;
});

var end = (0, _fp.curry)(function (stream, name, envelope) {
  stream.push({ type: 'plugin_end', ts: (0, _utils.now)(), plugin: name });
  return envelope;
});

var mark = (0, _fp.curry)(function (marker, envelope) {
  return (0, _envelope.fmapData)(_data2.default.concatOne({ _lf_markers: [marker] }), envelope);
});

var unitDefaults = (0, _envelope.fmapData)(_data2.default.concatOne(_data2.default.emptyOne()));

var hashData = (0, _envelope.fmapData)(_data2.default.hashOne);

var source = (0, _fp.curry)(function (name, envelope) {
  return (0, _envelope.fmapData)(_data2.default.concatOne({ _lf_source: name }), envelope);
});

/**
 * Create a runable littlefork object.
 *
 * Upon calling this function, you receive a runnable littlefork pipeline and
 * an observable stream. The littlefork pipeline is a function that can be
 * called without any arguments. It will return a promise that resolves to the
 * result of the pipeline run. The stream object is used to receive messages
 * during the pipeline run. It's currently mainly used for logging purposes,
 * but can be used for more as well.
 *
 * The stream sends messages with the following types:
 *
 * - `log_info`
 * - `log_debug`
 * - `log_error`
 * - `plugin_start`
 * - `plugin_end`
 *
 * @param {Object} config Configuration for a littlefork run.
 * @param {Array.<String>} queryIds A list of ids to query.
 * @returns {Array.<Function, Observable>} Return a tuple with a function that
 * runs the pipeline and an observable object, that receives messages during the
 * pipeline run. The observable object is BaconJS stream and has the full BaconJS
 * API available.
 * @example
 * const [run, stream] = runner(config, queryIds);
 *
 * stream.onValue(msg => {
 *   switch (msg.type) {
 *     case 'log_info': console.log(msg.msg); break;
 *     // ... other cases ...
 *     default: break;
 *   }
 * });
 *
 * run();
 */
var runner = (0, _fp.curry)(function (cfg, queries) {
  var _flow = (0, _fp.flow)([_plugins.list, _plugins.load])(),
      _flow2 = _slicedToArray(_flow, 2),
      plugins = _flow2[0],
      missing = _flow2[1];

  var stream = _baconjs2.default.Bus();
  var marker = (0, _shortid.generate)();

  // Check that all dependencies are installed.
  if (!(0, _fp.isEmpty)(missing)) {
    var msg = 'Missing the following modules: ' + (0, _fp.join)(', ', missing);
    throw new Error(msg);
  }

  // Make sure we have all requested plugins.
  var missingPlugins = (0, _fp.flow)([_fp.keys, (0, _fp.difference)(cfg.plugins)])(plugins);
  if (!(0, _fp.isEmpty)(missingPlugins)) {
    var _msg = 'Missing the following plugins: ' + (0, _fp.join)(', ', missingPlugins);
    throw new Error(_msg);
  }

  // The pipeline is a list of tuples, where the first element of the tuple
  // is a string indicating the name of the plugin, and the second element
  // is a function, that is the actual plugin.
  //     [['twitter_search', f1], ['mongodb_store', f2]]
  var pipeline = (0, _fp.flow)([(0, _fp.map)(function (p) {
    return plugins[p];
  }), (0, _fp.zip)(cfg.plugins)])(cfg.plugins);

  var log = {
    info: function info(msg) {
      return stream.push({ type: 'log_info', msg: msg });
    },
    error: function error(msg) {
      return stream.push({ type: 'log_error', msg: msg });
    },
    debug: function debug(msg) {
      return stream.push({ type: 'log_debug', msg: msg });
    }
  };

  var run = function run() {
    return (
      // Iterate over the pipeline and run each transformation in sequence.
      (0, _combinators.reduceP)(function (envelope, _ref) {
        var _ref2 = _slicedToArray(_ref, 2),
            name = _ref2[0],
            plugin = _ref2[1];

        return (0, _plugins.liftManyA2)([start(stream, name), plugin, unitDefaults, hashData, source(name), mark(marker), stats(stream, name), end(stream, name)], envelope, { log: log, cfg: (0, _fp.merge)({ marker: marker }, cfg) });
      }, (0, _envelope.concatQueries)(queries, (0, _envelope.empty)()), pipeline).catch(function (e) {
        return stream.error(e);
      }).tap(function () {
        return stream.end();
      })
    );
  };

  run.marker = marker;
  run.stream = stream;

  return run;
});

exports.default = runner;